

# function to create dose-response vector for one strain
# givens: dose pattern, replicates, parameters, error
# param vector is slope, lower, upper, ed50
myfpl.strain <- function(dose.levels, replicate, param, error) {
	slope = param[1]
	lower = param[2]
	upper = param[3]
	ed50 = param[4]
	dose = rep(dose.levels, each=replicate)
	response = lower + (upper-lower)/(1 + exp(slope*(log(dose)-log(ed50))))
	error = rnorm(length(dose), 0, error)
	response = response + error
	return(response)
}

# function to create dose response data for parents and CSSs
# givens: parent strain and CSS parameter differences as vectors
# also given doses vector, no.replicates, and error
# param vector is slope, lower, upper, ed50
# uses myfpl.strain(function)
# returns a wide data frame, dose is not a factor
myfpl.data <- function(dose.levels, replicate, error, p1, p2, l1, l2, l3, l4, l5) {
	l1 = l1 + p1
	l2 = l2 + p1
	l3 = l3 + p1
	l4 = l4 + p1
	l5 = l5 + p1
	y.p1 = myfpl.strain(dose.levels, replicate, p1, error)
	y.p2 = myfpl.strain(dose.levels, replicate, p2, error)
	y.l1 = myfpl.strain(dose.levels, replicate, l1, error)
	y.l2 = myfpl.strain(dose.levels, replicate, l2, error)
	y.l3 = myfpl.strain(dose.levels, replicate, l3, error)
	y.l4 = myfpl.strain(dose.levels, replicate, l4, error)
	y.l5 = myfpl.strain(dose.levels, replicate, l5, error)
	dose = rep(dose.levels, each=replicate)
	fpl.data = data.frame(cbind(dose, y.p1, y.p2, y.l1, y.l2, y.l3, y.l4, y.l5))
	return(fpl.data)
}


# function to reshape CSS dose-response data from wide to long
# pass and return a data frame
# need long format data for later analysis
wide.to.long <- function(mydata) {
	require(reshape2)
	mydata.long = melt(mydata, id=c("dose"), measure.vars=c("y.p1", "y.p2", "y.l1", "y.l2", "y.l3", "y.l4", "y.l5"), value.name="response", variable.name = "strain")
	mydata.long$row = 1:length(mydata.long[,1])
	mydata.long$strain = factor(mydata.long$strain)
	return(mydata.long)
}


# function to plot the data from a myfpl.data() data frame
# this is not essential to the simulation but useful to see the 
# effects of the parameters chosen (black is background, gold is donor, 
# purple are CSSs)
myfpl.plot <- function(mydata) {
	require(drc)
	fpl.result = drm(mydata[,4]~dose, data = mydata, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
	#print(summary(fpl.result))
	plot(fpl.result, col="purple", axes=FALSE, ylab="response", type="none", ylim=c(90,220))
	axis(1, at=dose.levels, labels=dose.levels)
	axis(2, at=c(100,120,140,160,180,200,220), labels=c(100,120,140,160,180,200,220))

	for (v in c(5:8)) {
		fpl.result = drm(mydata[,v]~dose, data = mydata, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
		#print(summary(fpl.result))
		plot(fpl.result, add=TRUE, col="purple", axes=FALSE, ylab="", type="none")
	}

	fpl.result = drm(mydata[,2]~dose, data = mydata, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
	#print(summary(fpl.result))
	plot(fpl.result, add=TRUE, col="black", lwd=2, axes=FALSE, ylab="", type="none")

	fpl.result = drm(mydata[,3]~dose, data = mydata, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
	#print(summary(fpl.result))
	plot(fpl.result, add=TRUE, col="orange", lwd=2, axes=FALSE, ylab="", type="none")
}

#make function to generate low, middle, & high dose parental data
#returns data in long format
#replicates are fixed at 8 animals per dose to maintain 24 animals total
#returns a data frame
parents3doses <- function(dose.levels, replicate=8, error, p1, p2, l1, l2, l3, l4, l5) {
	require(reshape2)
	mydata = myfpl.data(dose.levels, replicate=8, error, p1, p2, l1, l2, l3, l4, l5)
	myindex = which(mydata$dose %in% c(0.5, 4, 32))
	mydata = mydata[myindex, c(1:3)]
	mydata$row = c(1:length(mydata[,1]))
	mydata.long = melt(mydata, id=c("row", "dose"), measure.vars=c("y.p1", "y.p2"),  	value.name="response", variable.name = "strain")
	mydata.long$dose = factor(mydata.long$dose)
	mydata.long$strain = factor(mydata.long$strain)
	mydata.long$row = c(1:length(mydata.long[,1]))
	mydata.long$id = mydata.long$row
	mydata.long = mydata.long[,-1]
	return(mydata.long)
}

#make function to generate 8 doses by 3 replicates parental data
#returns data in long format
#replicates are fixed at 3 animals per dose to maintain 24 animals total
#returns a data frame
parents8doses <- function(dose.levels, replicate=3, error, p1, p2, l1, l2, l3, l4, l5) {
	require(reshape2)
	mydata = myfpl.data(dose.levels, replicate=3, error, p1, p2, l1, l2, l3, l4, l5)
	mydata = mydata[, c(1:3)]
	mydata$row = c(1:length(mydata[,1]))
	mydata.long = melt(mydata, id=c("row", "dose"), measure.vars=c("y.p1", "y.p2"),  	value.name="response", variable.name = "strain")
	mydata.long$dose = factor(mydata.long$dose)
	mydata.long$strain = factor(mydata.long$strain)
	mydata.long$row = c(1:length(mydata.long[,1]))
	mydata.long$id = mydata.long$row
	mydata.long = mydata.long[,-1]
	return(mydata.long)
}


# function to test parents by ANOVA
# set flag 1 if < 0.05 or 0 if > 0.05
# tests effects of strain and strain*dose above dose alone
# returns 1 if significant, 0 if not
interaction.test <- function(parents) {
	myanova = aov(formula = response ~ strain + dose + strain:dose, data = parents)
	#pvalue = summary(myanova)[[1]]["strain:dose",5] # just interaction
	myanova.reduced = aov(formula = response ~ dose, data = parents)
	myaov = anova(myanova.reduced, myanova)
	pvalue = myaov[2,6]
	if(pvalue < 0.05) flag=1 else flag=0
	return(flag)
}


# function to test strain:dose simple mean effects (SMEs) and 
# return dose level for smallest p-value
# pass dose pattern c(0.25, 0.5, 1, 2, 4, 8, 16, 32) and parents data
# pass other dose pattern as necessary! e.g., c(0.5, 4, 32)
# simple main effects in this context are like ttests but using the 
# overall ANOVA model MSE
sme.test <- function(dose.levels, parents) {
	mypvalues = numeric(length(dose.levels)) # to store p-values
	myanova = aov(formula = response ~ strain + dose + strain:dose, data = parents)
	mean.square.r = summary(myanova)[[1]]["Residuals",3]
	df.r = summary(myanova)[[1]]["Residuals",1]
	for(d in 1:length(dose.levels)) {
		myindex = parents$dose == dose.levels[d]
		mysubset = parents[myindex,]
		mysubset.aov = aov(formula = response ~ strain, data = mysubset)
		mean.square.m = summary(mysubset.aov)[[1]]["strain",3]
		df.m = summary(mysubset.aov)[[1]]["strain",1]
		mypvalue = 1 - pf(mean.square.m/mean.square.r, df.m, df.r)
		mypvalues[d] = mypvalue
	}
	myresults = data.frame(cbind(dose.levels, mypvalues))
	psort = order(myresults$mypvalues)
	myresults.sorted = myresults[psort,]
	mydose = myresults.sorted[1,1]
	return(mydose)
}

# function to test parental data with 4PL model
# set flag 1 if < 0.05 or 0 if > 0.05
# this tests inclusion of strain against a model of the pooled data (8 parameters vs 4)
# returns 1 if significant and 0 if not
parents.4pl = function(parents) {
	require(drc)
	mydata = parents
	mydata$dose = as.numeric(levels(mydata$dose))[mydata$dose] # drm doesn't want dose factor so coerce back to numeric
	mymodel.full = drm(response ~ dose, data = mydata, curveid = strain, fct = LL.4(names=c("slope", "lower", "upper", "ed50")) )
	mymodel.reduced = drm(response ~ dose, data = mydata, fct = LL.4(names=c("slope", "lower", "upper", "ed50")) )
	myanova = anova(mymodel.reduced, mymodel.full, details=FALSE)
	mypvalue = myanova[2,5]
	if(mypvalue < 0.05) flag=1 else flag=0
	return(flag)
}

# function to test background parental against a given CSS
# pass data frame and CSS name as a string
# return p-value
css.fpl <- function(mydata.long, css) {
	require(drc)
	myindex = which(mydata.long$strain %in% c("y.p1", css))
	mysubset = mydata.long[myindex,]
	mymodel.full = drm(response ~ dose, data = mysubset, curveid = strain, fct = LL.4(names=c("slope", "lower", "upper", "ed50")) )
	mymodel.reduced = drm(response ~ dose, data = mysubset, fct = LL.4(names=c("slope", "lower", "upper", "ed50")) )
	myanova = anova(mymodel.reduced, mymodel.full, details=FALSE)
	mypvalue = myanova[2,5]
	return(mypvalue)
}



#create function to do t-test at specific dose.level
#pass mydata, dose.level, css
#return p-value
css.ttest <- function(mydata.long, dose.level, css) {
	myindex = which(mydata.long$strain %in% c("y.p1", css))
	mysubset = mydata.long[myindex,]
	mysubset = mysubset[mysubset$dose==dose.level,]
	myttest = t.test(formula = response ~ strain, data=mysubset)
	mypvalue = myttest$p.value
	return(mypvalue)
}



#create function to do oneway ANOVA on P1 and CSS 1-5
#followed by Dunnett's posthoc tests; given a data set
#drop P2, do ANOVA, an return a vector of 5 Dunnett p-values
#pvalues are ordered by line number, first to P1 vs L1, etc.
dunnett.test <- function(mydata.long, dose.level) {
	require(multcomp)
	myindex = which(mydata.long$strain %in% c("y.p1", "y.l1", "y.l2", "y.l3", "y.l4", "y.l5"))
	mysubset = mydata.long[myindex,]
	mysubset = mysubset[mysubset$dose==dose.level,]
	myaov = aov(formula = response ~ strain, data = mysubset)
	mydunnett = summary(glht(myaov, linfct = mcp(strain = "Dunnett")))
	mypvalues.dunnett = numeric(5)
	mypvalues.dunnett = mydunnett[["test"]][["pvalues"]][1:5]
	return(mypvalues.dunnett)
}


#create function to do 4PL model on P1 and CSS 1-5
#drop p2, create five additional strain factor variables
#with P1 combined with each CSS
#fit full model and reduced 4PL models and do anova() test
#collect p-values into one vector
#apply multiple test correction to vector, p.adjust
#return p-values
all.css.fpl <- function(mydata.long) {
	require(drc)
	myindex = which(mydata.long$strain %in% c("y.p1", "y.l1", "y.l2", "y.l3", "y.l4", "y.l5"))
	mysubset = mydata.long[myindex,]
	mysubset$strain1 = mysubset$strain
	levels(mysubset$strain1) = c("y.p1l1", "y.p2", "y.p1l1", "y.l2", "y.l3", "y.l4", "y.l5")
	mysubset$strain2 = mysubset$strain
	levels(mysubset$strain2) = c("y.p1l2", "y.p2", "y.l1", "y.p1l2", "y.l3", "y.l4", "y.l5")
	mysubset$strain3 = mysubset$strain
	levels(mysubset$strain3) = c("y.p1l3", "y.p2", "y.l1", "y.l2", "y.p1l3", "y.l4", "y.l5")
	mysubset$strain4 = mysubset$strain
	levels(mysubset$strain4) = c("y.p1l4", "y.p2", "y.l1", "y.l2", "y.l3", "y.p1l4", "y.l5")
	mysubset$strain5 = mysubset$strain
	levels(mysubset$strain5) = c("y.p1l5", "y.p2", "y.l1", "y.l2", "y.l3", "y.l4", "y.p1l5")
	mymodel.full = drm(response ~ dose, data = mysubset, curveid = strain, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
	mymodel.reduced1 = drm(response ~ dose, data = mysubset, curveid = strain1, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
	mymodel.reduced2 = drm(response ~ dose, data = mysubset, curveid = strain2, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
	mymodel.reduced3 = drm(response ~ dose, data = mysubset, curveid = strain3, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
	mymodel.reduced4 = drm(response ~ dose, data = mysubset, curveid = strain4, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
	mymodel.reduced5 = drm(response ~ dose, data = mysubset, curveid = strain5, fct = LL.4(names=c("slope", "lower", "upper", "ed50")))
	myanova1 = anova(mymodel.reduced1, mymodel.full, details=FALSE)
	myanova2 = anova(mymodel.reduced2, mymodel.full, details=FALSE)
	myanova3 = anova(mymodel.reduced3, mymodel.full, details=FALSE)
	myanova4 = anova(mymodel.reduced4, mymodel.full, details=FALSE)
	myanova5 = anova(mymodel.reduced5, mymodel.full, details=FALSE)
	mypvalue1 = myanova1[2,5]
	mypvalue2 = myanova2[2,5]
	mypvalue3 = myanova3[2,5]
	mypvalue4 = myanova4[2,5]
	mypvalue5 = myanova5[2,5]
	mypvalues.fpl = c(mypvalue1, mypvalue2, mypvalue3, mypvalue4, mypvalue5)
	mypvalues.fpl.adj = p.adjust(mypvalues.fpl, method ="holm")
	return(mypvalues.fpl.adj)
}



